"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const compressible_1 = tslib_1.__importDefault(require("compressible"));
const iconv_lite_1 = tslib_1.__importDefault(require("iconv-lite"));
const async_enumerable_1 = require("@reactive-js/async-enumerable");
const http_1 = require("@reactive-js/http");
const node_1 = require("@reactive-js/node");
const pipe_1 = require("@reactive-js/pipe");
const httpContentEncoding_1 = require("./httpContentEncoding");
exports.encodeHttpContent = (contentBody, encoding, options) => {
    const { body, contentLength, contentEncodings } = contentBody;
    return contentLength === 0
        ? contentBody
        : Object.assign(Object.assign({}, contentBody), { body: pipe_1.pipe(body, node_1.transform(httpContentEncoding_1.createEncodingCompressTransform(encoding, options))), contentEncodings: [...contentEncodings, encoding], contentLength: -1 });
};
exports.decodeHttpContent = (contentBody, options) => {
    const { body, contentLength, contentEncodings } = contentBody;
    return contentEncodings.length === 0 || contentLength === 0
        ? contentBody
        : Object.assign(Object.assign({}, contentBody), { body: contentEncodings
                .map(encoding => httpContentEncoding_1.createEncodingDecompressTransform(encoding, options))
                .reduceRight((acc, decoder) => pipe_1.pipe(acc, node_1.transform(decoder)), body), contentEncodings: [] });
};
exports.createBufferHttpContent = (chunk, contentType) => ({
    body: async_enumerable_1.ofValueStream(chunk),
    contentEncodings: [],
    contentLength: chunk.length,
    contentType: typeof contentType === "string"
        ? http_1.parseMediaTypeOrThrow(contentType)
        : contentType,
});
exports.createReadableHttpContent = (factory, contentType, contentLength = -1) => ({
    body: node_1.createBufferStreamFromReadable(factory),
    contentEncodings: [],
    contentLength,
    contentType: typeof contentType === "string"
        ? http_1.parseMediaTypeOrThrow(contentType)
        : contentType,
});
exports.createStringHttpContent = (content, contentType) => {
    var _a;
    contentType =
        typeof contentType === "string"
            ? http_1.parseMediaTypeOrThrow(contentType)
            : contentType;
    const charset = (_a = contentType.params["charset"]) !== null && _a !== void 0 ? _a : "utf-8";
    const buffer = iconv_lite_1.default.encode(content, charset);
    return exports.createBufferHttpContent(buffer, contentType);
};
const compressionBlacklist = [
    "text/event-stream",
];
exports.contentIsCompressible = (content) => {
    const { type, subtype } = content.contentType;
    const mediaType = http_1.mediaTypeToString({ type, subtype, params: {} });
    return (!compressionBlacklist.includes(mediaType) &&
        (compressible_1.default(mediaType) || false));
};
//# sourceMappingURL=httpContent.js.map