import compressible from "compressible";
import iconv from "iconv-lite";
import { ofValueStream, } from "@reactive-js/async-enumerable";
import { mediaTypeToString, parseMediaTypeOrThrow, } from "@reactive-js/http";
import { transform, createBufferStreamFromReadable } from "@reactive-js/node";
import { pipe } from "@reactive-js/pipe";
import { createEncodingCompressTransform, createEncodingDecompressTransform, } from "./httpContentEncoding";
export const encodeHttpContent = (contentBody, encoding, options) => {
    const { body, contentLength, contentEncodings } = contentBody;
    return contentLength === 0
        ? contentBody
        : Object.assign(Object.assign({}, contentBody), { body: pipe(body, transform(createEncodingCompressTransform(encoding, options))), contentEncodings: [...contentEncodings, encoding], contentLength: -1 });
};
export const decodeHttpContent = (contentBody, options) => {
    const { body, contentLength, contentEncodings } = contentBody;
    return contentEncodings.length === 0 || contentLength === 0
        ? contentBody
        : Object.assign(Object.assign({}, contentBody), { body: contentEncodings
                .map(encoding => createEncodingDecompressTransform(encoding, options))
                .reduceRight((acc, decoder) => pipe(acc, transform(decoder)), body), contentEncodings: [] });
};
export const createBufferHttpContent = (chunk, contentType) => ({
    body: ofValueStream(chunk),
    contentEncodings: [],
    contentLength: chunk.length,
    contentType: typeof contentType === "string"
        ? parseMediaTypeOrThrow(contentType)
        : contentType,
});
export const createReadableHttpContent = (factory, contentType, contentLength = -1) => ({
    body: createBufferStreamFromReadable(factory),
    contentEncodings: [],
    contentLength,
    contentType: typeof contentType === "string"
        ? parseMediaTypeOrThrow(contentType)
        : contentType,
});
export const createStringHttpContent = (content, contentType) => {
    var _a;
    contentType =
        typeof contentType === "string"
            ? parseMediaTypeOrThrow(contentType)
            : contentType;
    const charset = (_a = contentType.params["charset"]) !== null && _a !== void 0 ? _a : "utf-8";
    const buffer = iconv.encode(content, charset);
    return createBufferHttpContent(buffer, contentType);
};
const compressionBlacklist = [
    "text/event-stream",
];
export const contentIsCompressible = (content) => {
    const { type, subtype } = content.contentType;
    const mediaType = mediaTypeToString({ type, subtype, params: {} });
    return (!compressionBlacklist.includes(mediaType) &&
        (compressible(mediaType) || false));
};
//# sourceMappingURL=httpContent.js.map